<?php
namespace MediaFiles;

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

class Pro_Settings {
	public function __construct() {
		add_action( 'admin_init', [$this, 'register_pro_settings'] );
		add_filter( 'Bwdmediafiles/post_types', [$this, 'enabled_post_types'], 10, 1 );

		add_action( 'Bwdmediafiles/settings/general/top', [$this, 'render_setting_post_types' ] );
		add_action( 'Bwdmediafiles/settings/general/top', [$this, 'render_setting_disable_for_files' ] );
		add_action( 'Bwdmediafiles/settings/general/top', [$this, 'render_load_for_logged_in_users' ] );

		add_action( 'Bwdmediafiles/settings/general/bottom', [$this, 'render_filesizes' ] );
		add_action( 'Bwdmediafiles/settings/general/bottom', [$this, 'render_featured_image_quick_edit' ] );
		add_action( 'Bwdmediafiles/settings/general/bottom', [$this, 'render_miscellaneous' ] );

		add_action( 'Bwdmediafiles/settings/permissions/bottom', [$this, 'render_setting_svg_support' ] );
	}

	/**
	 * Disable MediaFiles for files (post type: attachment)
	 */
	public function enabled_post_types( $post_types ) {
		$disable_for_files = get_option( 'bwdmfmx_mediafiles_disable_for_files', false );

		if ( $disable_for_files ) {
			$attachment_index = array_search( 'attachment', $post_types );

			if ( $attachment_index !== false ) {
				unset( $post_types[$attachment_index] );
			}
		}

		return $post_types;
	}
	

	/**
	 * Register PRO settings
	 */
	public function register_pro_settings() {
		// Tab: General
		register_setting( BWDMFMX_MEDIAFILES_SETTINGS_GROUP, 'bwdmfmx_mediafiles_post_types' );
		register_setting( BWDMFMX_MEDIAFILES_SETTINGS_GROUP, 'bwdmfmx_mediafiles_disable_for_files' );
		register_setting( BWDMFMX_MEDIAFILES_SETTINGS_GROUP, 'bwdmfmx_mediafiles_filesizes' );
		register_setting( BWDMFMX_MEDIAFILES_SETTINGS_GROUP, 'bwdmfmx_mediafiles_featured_image_quick_edit' );
		register_setting( BWDMFMX_MEDIAFILES_SETTINGS_GROUP, 'bwdmfmx_mediafiles_featured_image_column' );
		register_setting( BWDMFMX_MEDIAFILES_SETTINGS_GROUP, 'bwdmfmx_mediafiles_folder_name_in_title' );

		// @since 1.8
		register_setting( BWDMFMX_MEDIAFILES_SETTINGS_GROUP, 'bwdmfmx_mediafiles_load_wp_admin' );
		register_setting( BWDMFMX_MEDIAFILES_SETTINGS_GROUP, 'bwdmfmx_mediafiles_load_frontend' );

		// Tab: Permissions
		register_setting( BWDMFMX_MEDIAFILES_SETTINGS_GROUP, 'bwdmfmx_mediafiles_svg_support' );
		register_setting( BWDMFMX_MEDIAFILES_SETTINGS_GROUP, 'bwdmfmx_mediafiles_disable_svg_sanitization' );

		register_setting( BWDMFMX_MEDIAFILES_SETTINGS_GROUP, 'bwdmfmx_mediafiles_license_key' );
	}

	

	/**
	 * PRO setting: Post Types
	 */
	public function render_setting_post_types() { ?>
		<tr>
			<th><?php esc_html_e( 'Post Types', 'Bwdmediafiles' ); ?></th>
			<td>
				<fieldset>
					<?php
					$registered_post_types = get_post_types( ['show_ui' => true], 'objects' );
					$enabled_post_types = Settings::$enabled_post_types;

					// Folders for plugins (@since 1.6)
					$plugins_object = new \stdClass();
					$plugins_object->name = 'plugins';
					$plugins_object->label = esc_html__( 'Plugins', 'Bwdmediafiles' );
					$registered_post_types[] = $plugins_object;

					foreach ( $registered_post_types as $post_type ) {
						$post_type_name = $post_type->name;
						$post_type_label = $post_type->label;

						if ( in_array( $post_type_name, ['attachment', 'wp_block'] ) ) {
							continue;
						}
						?>
						<label for="<?php echo esc_attr( "bwdmfmx_mediafiles_post_types_$post_type_name" ); ?>">
						<input 
							name="bwdmfmx_mediafiles_post_types[]" 
							id="<?php echo esc_attr( "bwdmfmx_mediafiles_post_types_$post_type_name" ); ?>" 
							type="checkbox" 
							value="<?php echo esc_attr( $post_type_name ); ?>"
							<?php checked( in_array( $post_type_name, $enabled_post_types ) ); ?>>
						<?php echo $post_type->label; ?>
						</label>
						<br>
					<?php } ?>
				</fieldset>
			</td>
		</tr>
		<?php
	}

	/**
	 * PRO setting: Disable MediaFiles for files
	 */
	public function render_setting_disable_for_files() { ?>
		<tr>
			<th><?php esc_html_e( 'Disable in media library', 'Bwdmediafiles' ); ?></th>
			<td>
				<fieldset>
						<label for="bwdmfmx_mediafiles_disable_for_files">
						<input 
							name="bwdmfmx_mediafiles_disable_for_files" 
							id="bwdmfmx_mediafiles_disable_for_files" 
							type="checkbox" 
							value="1"
							<?php checked( get_option( 'bwdmfmx_mediafiles_disable_for_files', false ) ); ?>>
							<?php esc_html_e( 'Disable in media library', 'Bwdmediafiles' ); ?>
						</label>
				</fieldset>
			</td>
		</tr>
		<?php
	}

	/**
	 * PRO setting: Enforce loading
	 * 
	 * Ensure MediaFiles is loaded wherever user needs it.
	 * 
	 * Like on the frontend when using a page builder that is not natively supported by MediaFiles.
	 * 
	 * @since 1.8
	 */
	public function render_load_for_logged_in_users() { 
		?>
		<tr>
			<th><?php esc_html_e( 'Enforce loading', 'Bwdmediafiles' ); ?></th>
			<td>
				<fieldset>
					<label for="bwdmfmx_mediafiles_load_wp_admin">
						<input 
							name="bwdmfmx_mediafiles_load_wp_admin" 
							id="bwdmfmx_mediafiles_load_wp_admin" 
							type="checkbox" 
							value="1"
							<?php checked( get_option( 'bwdmfmx_mediafiles_load_wp_admin', false ) ); ?>>
							<?php echo 'wp-admin'; ?>
					</label>
					<br>

					<label for="bwdmfmx_mediafiles_load_frontend">
						<input 
							name="bwdmfmx_mediafiles_load_frontend" 
							id="bwdmfmx_mediafiles_load_frontend" 
							type="checkbox" 
							value="1"
							<?php checked( get_option( 'bwdmfmx_mediafiles_load_frontend', false ) ); ?>>
							<?php echo 'Frontend'; ?>
					</label>
					<br>

					<p class="description"><?php esc_html_e( 'Enforce loading MediaFiles in the wp-admin or frontend (logged-in useres only), select the loading location above.', 'Bwdmediafiles' ); ?></p>
				</fieldset>
			</td>
		</tr>
		<?php
	}

	/**
	 * PRO setting: Sort attachments by filesize
	 */
	public function render_filesizes() { ?>
		<tr>
			<th><?php esc_html_e( 'Sort by file size', 'Bwdmediafiles' ); ?></th>
			<td>
				<fieldset>
				<?php $filesize_enabled = Settings::$filesizes; ?>
				<label for="bwdmfmx_mediafiles_filesizes">
					<input type="checkbox" name="bwdmfmx_mediafiles_filesizes" id="bwdmfmx_mediafiles_filesizes" value="1" <?php checked( $filesize_enabled, true, true ); ?>>
					<?php esc_html_e( 'Sort by file size', 'Bwdmediafiles' ); ?>
				</label>
				<br>
				<p class="description"><?php esc_html_e( 'When enabled you can sort your attachments by file size.', 'Bwdmediafiles' ); ?></p>
				</fieldset>

				<?php
				if ( $filesize_enabled ) {
					$attachment_ids = get_posts( [
						'post_type'      => 'attachment',
						'posts_per_page' => -1,
						'post_status'    => 'any',
						'fields'         => 'ids',
						'order'          => 'ASC',
					] );
					
					echo '<button id="bwdmfmx-mediafiles-regenerate-filesizes" class="button button-secondary" data-attachment-ids="' . implode( ',', $attachment_ids ) . '">' . esc_html__( 'Regenerate file sizes', 'Bwdmediafiles' ) . '</button>';

					echo '<ul id="bwdmfmx-mediafiles-regenerate-filesizes-results"></ul>';
				}
				?>
			</td>
		</tr>
		<?php
	}

	/**
	 * PRO setting: Featured Image (via "Quick Edit", Preview column)
	 */
	public function render_featured_image_quick_edit() { ?>
		<tr>
			<th><?php esc_html_e( 'Featured image', 'Bwdmediafiles' ); ?></th>
			<td>
				<fieldset>
					<?php $featured_image_quick_edit = get_option( 'bwdmfmx_mediafiles_featured_image_quick_edit', false ); ?>
					<label for="bwdmfmx_mediafiles_featured_image_quick_edit">
						<input type="checkbox" name="bwdmfmx_mediafiles_featured_image_quick_edit" id="bwdmfmx_mediafiles_featured_image_quick_edit" value="1" <?php checked( $featured_image_quick_edit, true, true ); ?>>
						<?php esc_html_e( 'Set featured image via "Quick edit"', 'Bwdmediafiles' ); ?>
					</label>
				</fieldset>

				</fieldset>
					<?php $featured_image_column = get_option( 'bwdmfmx_mediafiles_featured_image_column', false ); ?>
					<label for="bwdmfmx_mediafiles_featured_image_column">
						<input type="checkbox" name="bwdmfmx_mediafiles_featured_image_column" id="bwdmfmx_mediafiles_featured_image_column" value="1" <?php checked( $featured_image_column, true, true ); ?>>
						<?php esc_html_e( 'Show featured image preview column', 'Bwdmediafiles' ); ?>
					</label>
				</fieldset>
			</td>
		</tr>
		<?php
	}

	/**
	 * PRO setting: Miscellaneous
	 * 
	 * - Add folder name as 'title' attribute
	 * 
	 * @since 1.6
	 */
	public function render_miscellaneous() { ?>
		<tr>
			<th><?php esc_html_e( 'Miscellaneous', 'Bwdmediafiles' ); ?></th>
			<td>
				<fieldset>
				<label for="bwdmfmx_mediafiles_folder_name_in_title">
					<input type="checkbox" name="bwdmfmx_mediafiles_folder_name_in_title" id="bwdmfmx_mediafiles_folder_name_in_title" value="1" <?php checked( Settings::$folder_name_in_title, true, true ); ?>>
					<?php esc_html_e( 'View full folder name on hover', 'Bwdmediafiles' ); ?>
				</label>
				</fieldset>
			</td>
		</tr>
		<?php
	}

	/**
	 * PRO setting: SVG support & sanitization
	 */
	public function render_setting_svg_support() {
		?>
		<tr>
			<th>
				<?php esc_html_e( 'SVG support', 'Bwdmediafiles' ); ?>
				<p class="description">
					<?php esc_html_e( 'Enable/disable SVG upload, preview & sanitization per user role. Set SVG support for individual user on user profile page.', 'Bwdmediafiles' ); ?><br><br>
					<?php esc_html_e( 'Upload SVG files only from trusted sources as they can contain and execute malicious XML code.', 'Bwdmediafiles' ); ?>
				</p>
			</th>
			
			<td>
				<?php
				global $wp_roles;
				$user_roles = $wp_roles->role_names;
				$svg_support_per_role = get_option( 'bwdmfmx_mediafiles_svg_support', [] );

				foreach ( $user_roles as $name => $label ) {
					$svg_support = ! empty( $svg_support_per_role[$name] ) ? $svg_support_per_role[$name] : '';

					if ( $name === 'administrator' ) {
						$svg_support = '1';
					}

					echo '<section data-user-role="' . $name . '">';
					echo '<label for="bwdmfmx_mediafiles_svg_support_' . $name . '">' . $label . '</label>';
					echo '<select id="bwdmfmx_mediafiles_svg_support_' . $name . '" name="bwdmfmx_mediafiles_svg_support[' . $name . ']" ' . ( $name === 'administrator' ? 'disabled' : '' ) . '>';
					echo '<option value="" ' . selected( $svg_support === '' ) . '>' . esc_html__( 'Disabled', 'Bwdmediafiles' ) . '</option>';
					echo '<option value="1" ' . selected( $svg_support === '1' ) . '>' . esc_html__( 'Enabled', 'Bwdmediafiles' ) . '</option>';
					echo '</select>';
					echo '</section>';
				}
				?>
			</td>
		</tr>

		<tr>
			<th>
				<?php esc_html_e( 'Disable SVG sanitization', 'Bwdmediafiles' ); ?>
			</th>

			<td>
				<?php 
				$disable_svg_sanitization = get_option( 'bwdmfmx_mediafiles_disable_svg_sanitization', false );
				?>
				
				<fieldset>
					<label for="bwdmfmx_mediafiles_disable_svg_sanitization">
						<input type="checkbox" name="bwdmfmx_mediafiles_disable_svg_sanitization" id="bwdmfmx_mediafiles_disable_svg_sanitization" value="1" <?php checked( $disable_svg_sanitization, true, true ); ?>>
						<?php esc_html_e( 'Disable SVG sanitization', 'Bwdmediafiles' ); ?>
					</label>
					<br>
					<p class="description">
						<?php esc_html_e( 'Disable built-in SVG sanitization if you experience issues when uploading SVG files.', 'Bwdmediafiles' ); ?><br><br>
					</p>
				</fieldset>
			</td>
		</tr>
		<?php
	}

	

	
}